@@ -1,10 +1,9 @@
 package rust
 
 import (
-	"github.com/anchore/grype/grype/distro"
 	"github.com/anchore/grype/grype/match"
+	"github.com/anchore/grype/grype/matcher/internal"
 	"github.com/anchore/grype/grype/pkg"
-	"github.com/anchore/grype/grype/search"
 	"github.com/anchore/grype/grype/vulnerability"
 	syftPkg "github.com/anchore/syft/syft/pkg"
 )
@@ -31,10 +30,6 @@ func (m *Matcher) Type() match.MatcherType {
 	return match.RustMatcher
 }
 
-func (m *Matcher) Match(store vulnerability.Provider, d *distro.Distro, p pkg.Package) ([]match.Match, error) {
-	criteria := search.CommonCriteria
-	if m.cfg.UseCPEs {
-		criteria = append(criteria, search.ByCPE)
-	}
-	return search.ByCriteria(store, d, p, m.Type(), criteria...)
+func (m *Matcher) Match(store vulnerability.Provider, p pkg.Package) ([]match.Match, []match.IgnoredMatch, error) {
+	return internal.MatchPackageByEcosystemAndCPEs(store, p, m.Type(), m.cfg.UseCPEs)
 }
