@@ -68,6 +68,36 @@ func TestNew(t *testing.T) {
 				},
 			},
 		},
+		{
+			name: "dpkg archive with source info",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.DpkgArchiveEntry{
+					Package:       "pkg-info",
+					Source:        "src-info",
+					Version:       "version-info",
+					SourceVersion: "src-version-info",
+					Architecture:  "arch-info",
+					Maintainer:    "maintainer-info",
+					InstalledSize: 10,
+					Files: []syftPkg.DpkgFileRecord{
+						{
+							Path: "path-info",
+							Digest: &file.Digest{
+								Algorithm: "algo-info",
+								Value:     "digest-info",
+							},
+							IsConfigFile: true,
+						},
+					},
+				},
+			},
+			upstreams: []UpstreamPackage{
+				{
+					Name:    "src-info",
+					Version: "src-version-info",
+				},
+			},
+		},
 		{
 			name: "rpm archive with source info",
 			syftPkg: syftPkg.Package{
@@ -160,10 +190,10 @@ func TestNew(t *testing.T) {
 				Name: "sqlite",
 				Metadata: syftPkg.RpmArchive{
 					SourceRpm:       "sqlite-3.26.0-6.el8.src.rpm",
-					ModularityLabel: "abc:2",
+					ModularityLabel: strRef("abc:2"),
 				},
 			},
-			metadata: RpmMetadata{ModularityLabel: "abc:2"},
+			metadata: RpmMetadata{ModularityLabel: strRef("abc:2")},
 		},
 		{
 			name: "java pkg",
@@ -171,12 +201,18 @@ func TestNew(t *testing.T) {
 				Metadata: syftPkg.JavaArchive{
 					VirtualPath: "virtual-path-info",
 					Manifest: &syftPkg.JavaManifest{
-						Main: map[string]string{
-							"Name": "main-section-name-info",
+						Main: syftPkg.KeyValues{
+							{
+								Key:   "Name",
+								Value: "main-section-name-info",
+							},
 						},
-						NamedSections: map[string]map[string]string{
-							"named-section": {
-								"named-section-key": "named-section-value",
+						Sections: []syftPkg.KeyValues{
+							{
+								{
+									Key:   "named-section-key",
+									Value: "named-section-value",
+								},
 							},
 						},
 					},
@@ -284,18 +320,27 @@ func TestNew(t *testing.T) {
 				},
 			},
 		},
+		{
+			name: "github-actions-use-statement",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.GitHubActionsUseStatement{
+					Value:   "a",
+					Comment: "a",
+				},
+			},
+		},
 		{
 			name: "golang-metadata",
 			syftPkg: syftPkg.Package{
 				Metadata: syftPkg.GolangBinaryBuildinfoEntry{
-					BuildSettings:     map[string]string{},
+					BuildSettings:     syftPkg.KeyValues{},
 					GoCompiledVersion: "1.0.0",
 					H1Digest:          "a",
 					MainModule:        "myMainModule",
 				},
 			},
 			metadata: GolangBinMetadata{
-				BuildSettings:     map[string]string{},
+				BuildSettings:     syftPkg.KeyValues{},
 				GoCompiledVersion: "1.0.0",
 				H1Digest:          "a",
 				MainModule:        "myMainModule",
@@ -360,19 +405,34 @@ func TestNew(t *testing.T) {
 			},
 		},
 		{
-			name: "cpp conan lock metadata",
+			name: "cpp conan v1 lock metadata",
 			syftPkg: syftPkg.Package{
-				Metadata: syftPkg.ConanLockEntry{
+				Metadata: syftPkg.ConanV1LockEntry{
 					Ref: "zlib/1.2.12",
-					Options: map[string]string{
-						"fPIC":   "True",
-						"shared": "False",
+					Options: syftPkg.KeyValues{
+						{
+							Key:   "fPIC",
+							Value: "True",
+						},
+						{
+							Key:   "shared",
+							Value: "false",
+						},
 					},
 					Path:    "all/conanfile.py",
 					Context: "host",
 				},
 			},
 		},
+		{
+			name: "cpp conan v2 lock metadata",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.ConanV2LockEntry{
+					Ref:       "zlib/1.2.12",
+					PackageID: "some-id",
+				},
+			},
+		},
 		{
 			name: "cocoapods cocoapods-metadata",
 			syftPkg: syftPkg.Package{
@@ -560,6 +620,23 @@ func TestNew(t *testing.T) {
 				},
 			},
 		},
+		{
+			name: "swipl-pack-entry",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.SwiplPackEntry{
+					Name:          "a",
+					Version:       "a",
+					Author:        "a",
+					AuthorEmail:   "a",
+					Packager:      "a",
+					PackagerEmail: "a",
+					Homepage:      "a",
+					Dependencies: []string{
+						"a",
+					},
+				},
+			},
+		},
 		{
 			name: "conaninfo-entry",
 			syftPkg: syftPkg.Package{
@@ -579,6 +656,155 @@ func TestNew(t *testing.T) {
 				},
 			},
 		},
+		{
+			name: "python-poetry-lock-entry",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.PythonPoetryLockEntry{Index: "some-index"},
+			},
+		},
+		{
+			name: "yarn-lock-entry",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.YarnLockEntry{
+					Resolved:  "some-resolution",
+					Integrity: "some-digest",
+				},
+			},
+		},
+		{
+			name: "wordpress-plugin-entry",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.WordpressPluginEntry{
+					PluginInstallDirectory: "a",
+					Author:                 "a",
+					AuthorURI:              "a",
+				},
+			},
+		},
+		{
+			name: "elf-binary-package",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.ELFBinaryPackageNoteJSONPayload{
+					Type:       "a",
+					Vendor:     "a",
+					System:     "a",
+					SourceRepo: "a",
+					Commit:     "a",
+				},
+			},
+		},
+		{
+			name: "Php-pecl-entry",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.PhpPeclEntry{
+					Name:    "a",
+					Version: "a",
+					License: []string{"a"},
+				},
+			},
+		},
+		{
+			name: "lua-rocks-entry",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.LuaRocksPackage{
+					Name:         "a",
+					Version:      "a",
+					License:      "a",
+					Homepage:     "a",
+					Description:  "a",
+					URL:          "a",
+					Dependencies: map[string]string{"b": "c"},
+				},
+			},
+		},
+		{
+			name: "ocaml-entry",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.OpamPackage{
+					Name:         "a",
+					Version:      "a",
+					Licenses:     []string{"a"},
+					URL:          "a",
+					Checksums:    []string{"a"},
+					Homepage:     "a",
+					Dependencies: []string{"a"},
+				},
+			},
+		},
+		{
+			name: "jvm-installation-entry",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.JavaVMInstallation{
+					Release: syftPkg.JavaVMRelease{
+						Implementor:        "a",
+						ImplementorVersion: "a",
+						JavaRuntimeVersion: "b",
+						JavaVersion:        "c",
+						JavaVersionDate:    "a",
+						Libc:               "a",
+						Modules:            []string{"a"},
+						OsArch:             "a",
+						OsName:             "a",
+						OsVersion:          "a",
+						Source:             "a",
+						BuildSource:        "a",
+						BuildSourceRepo:    "a",
+						SourceRepo:         "a",
+						FullVersion:        "d",
+						SemanticVersion:    "e",
+						BuildInfo:          "a",
+						JvmVariant:         "a",
+						JvmVersion:         "a",
+						ImageType:          "a",
+						BuildType:          "a",
+					},
+					Files: []string{"a"},
+				},
+			},
+			metadata: JavaVMInstallationMetadata{
+				Release: JavaVMReleaseMetadata{
+					JavaRuntimeVersion: "b",
+					JavaVersion:        "c",
+					FullVersion:        "d",
+					SemanticVersion:    "e",
+				},
+			},
+		},
+		{
+			name: "dotnet-package-lock-entry",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.DotnetPackagesLockEntry{
+					Name:        "AutoMapper",
+					Version:     "13.0.1",
+					ContentHash: "/Fx1SbJ16qS7dU4i604Sle+U9VLX+WSNVJggk6MupKVkYvvBm4XqYaeFuf67diHefHKHs50uQIS2YEDFhPCakQ==",
+					Type:        "Direct",
+				},
+			},
+		},
+		{
+			name: "bitnami-sbom-entry",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.BitnamiSBOMEntry{
+					Name:    "a",
+					Version: "1",
+				},
+			},
+		},
+		{
+			name: "terraform-lock-provider-entry",
+			syftPkg: syftPkg.Package{
+				Metadata: syftPkg.TerraformLockProviderEntry{
+					URL:         "registry.terraform.io/hashicorp/aws",
+					Version:     "5.72.1",
+					Constraints: "> 5.72.0",
+					Hashes: []string{
+						"h1:jhd5O5o0CfZCNEwwN0EiDAzb7ApuFrtxJqa6HXW4EKE=",
+						"zh:0dea6843836e926d33469b48b948744079023816d16a2ff7666bcfb6aa3522d4",
+						"zh:195fa9513f75800a0d62797ebec75ee73e9b8c28d713fe9b63d3b1d1eec129b3",
+					},
+				},
+			},
+		},
 	}
 
 	// capture each observed metadata type, we should see all of them relate to what syft provides by the end of testing
@@ -806,7 +1032,7 @@ func catalogWithOverlaps(packages []string, overlaps []string) *sbom.SBOM {
 		pkgs = append(pkgs, p)
 	}
 
-	for _, overlap := range overlaps {
+	for i, overlap := range overlaps {
 		parts := strings.Split(overlap, "->")
 		if len(parts) < 2 {
 			panic("invalid overlap, use -> to specify, e.g.: pkg1->pkg2")
@@ -814,11 +1040,24 @@ func catalogWithOverlaps(packages []string, overlaps []string) *sbom.SBOM {
 		from := toPkg(parts[0])
 		to := toPkg(parts[1])
 
-		relationships = append(relationships, artifact.Relationship{
-			From: from,
-			To:   to,
-			Type: artifact.OwnershipByFileOverlapRelationship,
-		})
+		// The catalog will type check whether To or From is a pkg.Package or a *pkg.Package.
+		// Previously, there was a bug where Grype assumed that From was always a pkg.Package.
+		// Therefore, intentionally mix pointer and non-pointer packages to prevent Grype from
+		// assuming which is which again. (The correct usage, calling catalog.Package, always
+		// returns a *pkg.Package, and doesn't rely on any type assertion.)
+		if i%2 == 0 {
+			relationships = append(relationships, artifact.Relationship{
+				From: &from,
+				To:   &to,
+				Type: artifact.OwnershipByFileOverlapRelationship,
+			})
+		} else {
+			relationships = append(relationships, artifact.Relationship{
+				From: from,
+				To:   to,
+				Type: artifact.OwnershipByFileOverlapRelationship,
+			})
+		}
 	}
 
 	catalog := syftPkg.NewCollection(pkgs...)
@@ -837,3 +1076,7 @@ func withDistro(s *sbom.SBOM, id string) *sbom.SBOM {
 	}
 	return s
 }
+
+func strRef(s string) *string {
+	return &s
+}
