@@ -14,21 +14,51 @@ type golangVersion struct {
 	semVer *hashiVer.Version
 }
 
-func (g golangVersion) Compare(version *Version) (int, error) {
-	if version.Format != GolangFormat {
-		return -1, fmt.Errorf("cannot compare %v to golang version", version.Format)
-	}
-	if version.rich.golangVersion == nil {
-		return -1, fmt.Errorf("cannot compare version with nil golang version to golang version")
-	}
-	if version.rich.golangVersion.raw == g.raw {
-		return 0, nil
-	}
-	if version.rich.golangVersion.raw == "(devel)" {
-		return -1, fmt.Errorf("cannot compare %s with %s", g.raw, version.rich.golangVersion.raw)
+func newGolangVersion(v string) (*golangVersion, error) {
+	if v == "(devel)" {
+		return nil, ErrUnsupportedVersion
 	}
 
-	return version.rich.golangVersion.compare(g), nil
+	// Invalid Semver fix ups
+
+	// go stdlib is reported by syft as a go package with version like "go1.24.1"
+	// other versions have "v" as a prefix, which the semver lib handles automatically
+	fixedUp := strings.TrimPrefix(v, "go")
+
+	// go1.24 creates non-dot separated build metadata fields, e.g. +incompatible+dirty
+	// Fix up as per semver spec
+	before, after, found := strings.Cut(fixedUp, "+")
+	if found {
+		fixedUp = before + "+" + strings.ReplaceAll(after, "+", ".")
+	}
+
+	semver, err := hashiVer.NewSemver(fixedUp)
+	if err != nil {
+		return nil, err
+	}
+	return &golangVersion{
+		raw:    v,
+		semVer: semver,
+	}, nil
+}
+
+func (g golangVersion) Compare(other *Version) (int, error) {
+	other, err := finalizeComparisonVersion(other, GolangFormat)
+	if err != nil {
+		return -1, err
+	}
+
+	if other.rich.golangVersion == nil {
+		return -1, fmt.Errorf("cannot compare version with nil golang version to golang version")
+	}
+	if other.rich.golangVersion.raw == g.raw {
+		return 0, nil
+	}
+	if other.rich.golangVersion.raw == "(devel)" {
+		return -1, fmt.Errorf("cannot compare %s with %s", g.raw, other.rich.golangVersion.raw)
+	}
+
+	return other.rich.golangVersion.compare(g), nil
 }
 
 func (g golangVersion) compare(o golangVersion) int {
@@ -43,16 +73,3 @@ func (g golangVersion) compare(o golangVersion) int {
 		return strings.Compare(g.raw, o.raw)
 	}
 }
-
-func newGolangVersion(v string) (*golangVersion, error) {
-	// go stdlib is reported by syft as a go package with version like "go1.24.1"
-	// other versions have "v" as a prefix, which the semver lib handles automatically
-	semver, err := hashiVer.NewSemver(strings.TrimPrefix(v, "go"))
-	if err != nil {
-		return nil, err
-	}
-	return &golangVersion{
-		raw:    v,
-		semVer: semver,
-	}, nil
-}
