@@ -1,21 +1,20 @@
 package grype
 
 import (
+	"errors"
 	"fmt"
-	"slices"
+	"runtime/debug"
 	"strings"
 
 	"github.com/wagoodman/go-partybus"
 	"github.com/wagoodman/go-progress"
 
-	v5 "github.com/anchore/grype/grype/db/v5"
-	"github.com/anchore/grype/grype/db/v5/matcher"
-	"github.com/anchore/grype/grype/db/v5/matcher/stock"
 	"github.com/anchore/grype/grype/distro"
 	"github.com/anchore/grype/grype/event"
 	"github.com/anchore/grype/grype/event/monitor"
 	"github.com/anchore/grype/grype/grypeerr"
 	"github.com/anchore/grype/grype/match"
+	"github.com/anchore/grype/grype/matcher/stock"
 	"github.com/anchore/grype/grype/pkg"
 	"github.com/anchore/grype/grype/vex"
 	"github.com/anchore/grype/grype/vulnerability"
@@ -31,19 +30,13 @@ const (
 )
 
 type VulnerabilityMatcher struct {
-	Store          v5.ProviderStore
-	Matchers       []matcher.Matcher
-	IgnoreRules    []match.IgnoreRule
-	FailSeverity   *vulnerability.Severity
-	NormalizeByCVE bool
-	VexProcessor   *vex.Processor
-}
-
-func DefaultVulnerabilityMatcher(store v5.ProviderStore) *VulnerabilityMatcher {
-	return &VulnerabilityMatcher{
-		Store:    store,
-		Matchers: matcher.NewDefaultMatchers(matcher.Config{}),
-	}
+	VulnerabilityProvider vulnerability.Provider
+	ExclusionProvider     match.ExclusionProvider
+	Matchers              []match.Matcher
+	IgnoreRules           []match.IgnoreRule
+	FailSeverity          *vulnerability.Severity
+	NormalizeByCVE        bool
+	VexProcessor          *vex.Processor
 }
 
 func (m *VulnerabilityMatcher) FailAtOrAboveSeverity(severity *vulnerability.Severity) *VulnerabilityMatcher {
@@ -51,7 +44,7 @@ func (m *VulnerabilityMatcher) FailAtOrAboveSeverity(severity *vulnerability.Sev
 	return m
 }
 
-func (m *VulnerabilityMatcher) WithMatchers(matchers []matcher.Matcher) *VulnerabilityMatcher {
+func (m *VulnerabilityMatcher) WithMatchers(matchers []match.Matcher) *VulnerabilityMatcher {
 	m.Matchers = matchers
 	return m
 }
@@ -74,6 +67,7 @@ func (m *VulnerabilityMatcher) FindMatches(pkgs []pkg.Package, context pkg.Conte
 
 	remainingMatches, ignoredMatches, err = m.findDBMatches(pkgs, context, progressMonitor)
 	if err != nil {
+		err = fmt.Errorf("unable to find matches against vulnerability database: %w", err)
 		return remainingMatches, ignoredMatches, err
 	}
 
@@ -83,7 +77,7 @@ func (m *VulnerabilityMatcher) FindMatches(pkgs []pkg.Package, context pkg.Conte
 		return remainingMatches, ignoredMatches, err
 	}
 
-	if m.FailSeverity != nil && HasSeverityAtOrAbove(m.Store, *m.FailSeverity, *remainingMatches) {
+	if m.FailSeverity != nil && hasSeverityAtOrAbove(m.VulnerabilityProvider, *m.FailSeverity, *remainingMatches) {
 		err = grypeerr.ErrAboveSeverityThreshold
 		return remainingMatches, ignoredMatches, err
 	}
@@ -101,7 +95,13 @@ func (m *VulnerabilityMatcher) findDBMatches(pkgs []pkg.Package, context pkg.Con
 	log.Trace("finding matches against DB")
 	matches, err := m.searchDBForMatches(context.Distro, pkgs, progressMonitor)
 	if err != nil {
-		return nil, nil, fmt.Errorf("unable to find matches in DB: %w", err)
+		if match.IsFatalError(err) {
+			return nil, nil, err
+		}
+
+		// other errors returned from matchers during searchDBForMatches were being
+		// logged and not returned, so just log them here
+		log.WithFields("error", err).Debug("error(s) returned from searchDBForMatches")
 	}
 
 	matches, ignoredMatches = m.applyIgnoreRules(matches)
@@ -137,17 +137,19 @@ func (m *VulnerabilityMatcher) mergeIgnoredMatches(allIgnoredMatches ...[]match.
 	return out
 }
 
+//nolint:funlen
 func (m *VulnerabilityMatcher) searchDBForMatches(
 	release *linux.Release,
 	packages []pkg.Package,
 	progressMonitor *monitorWriter,
 ) (match.Matches, error) {
-	var err error
-	res := match.NewMatches()
+	var allMatches []match.Match
+	var allIgnored []match.IgnoredMatch
 	matcherIndex, defaultMatcher := newMatcherIndex(m.Matchers)
 
 	var d *distro.Distro
 	if release != nil {
+		var err error
 		d, err = distro.NewFromRelease(*release)
 		if err != nil {
 			log.Warnf("unable to determine linux distribution: %+v", err)
@@ -158,137 +160,78 @@ func (m *VulnerabilityMatcher) searchDBForMatches(
 		}
 	}
 
-	distroFalsePositivesByLocationPath := make(map[string][]string)
-	if d != nil && (d.Type == distro.Wolfi || d.Type == distro.Chainguard || d.Type == distro.Alpine) {
-		distroFalsePositivesByLocationPath, err = indexFalsePositivesByLocation(d, packages, m.Store)
-		if err != nil {
-			return match.Matches{}, err
-		}
-	}
-
 	if defaultMatcher == nil {
 		defaultMatcher = stock.NewStockMatcher(stock.MatcherConfig{UseCPEs: true})
 	}
+
+	var matcherErrs []error
 	for _, p := range packages {
 		progressMonitor.PackagesProcessed.Increment()
 		log.WithFields("package", displayPackage(p)).Trace("searching for vulnerability matches")
 
+		// if there is no distro set, default to global distro
+		orig := p.Distro
+		if orig == nil {
+			p.Distro = d
+		}
+
 		matchAgainst, ok := matcherIndex[p.Type]
 		if !ok {
-			matchAgainst = []matcher.Matcher{defaultMatcher}
+			matchAgainst = []match.Matcher{defaultMatcher}
 		}
 		for _, theMatcher := range matchAgainst {
-			matches, err := theMatcher.Match(m.Store, d, p)
+			matches, ignoredMatches, err := callMatcherSafely(theMatcher, m.VulnerabilityProvider, p)
 			if err != nil {
-				log.WithFields("error", err, "package", displayPackage(p)).Warn("matcher failed")
-				continue
+				if match.IsFatalError(err) {
+					return match.Matches{}, err
+				}
+
+				log.WithFields("error", err, "package", displayPackage(p)).Warn("matcher returned error")
+				matcherErrs = append(matcherErrs, err)
 			}
 
-			matches = filterMatchesUsingDistroFalsePositives(matches, distroFalsePositivesByLocationPath)
+			allIgnored = append(allIgnored, ignoredMatches...)
 
 			// Filter out matches based on records in the database exclusion table and hard-coded rules
-			filtered, dropped := match.ApplyExplicitIgnoreRules(m.Store, match.NewMatches(matches...))
+			filtered, dropped := match.ApplyExplicitIgnoreRules(m.ExclusionProvider, match.NewMatches(matches...))
 
 			additionalMatches := filtered.Sorted()
 			logPackageMatches(p, additionalMatches)
 			logExplicitDroppedPackageMatches(p, dropped)
-			res.Add(additionalMatches...)
+			allMatches = append(allMatches, additionalMatches...)
 
 			progressMonitor.MatchesDiscovered.Add(int64(len(additionalMatches)))
 
 			// note: there is a difference between "ignore" and "dropped" matches.
 			// ignored: matches that are filtered out due to user-provided ignore rules
 			// dropped: matches that are filtered out due to hard-coded rules
-			updateVulnerabilityList(progressMonitor, additionalMatches, nil, dropped, m.Store)
+			updateVulnerabilityList(progressMonitor, additionalMatches, nil, dropped, m.VulnerabilityProvider)
 		}
+
+		p.Distro = orig
 	}
 
-	return res, nil
+	// apply ignores based on matchers returning ignore rules
+	filtered, dropped := match.ApplyIgnoreFilters(allMatches, ignoredMatchFilter(allIgnored))
+	logIgnoredMatches(dropped)
+
+	// get deduplicated set of matches
+	res := match.NewMatches(filtered...)
+
+	// update the total discovered matches after removing all duplicates and ignores
+	progressMonitor.MatchesDiscovered.Set(int64(res.Count()))
+
+	return res, errors.Join(matcherErrs...)
 }
 
-func indexFalsePositivesByLocation(
-	d *distro.Distro,
-	packages []pkg.Package,
-	s v5.ProviderStore,
-) (map[string][]string, error) {
-	distroFalsePositivesByLocationPath := make(map[string][]string)
-
-	for _, p := range packages {
-		falsePositivesByLocation, err := getDistroFalsePositivesByLocation(s, d, p)
-		if err != nil {
-			return nil, err
+func callMatcherSafely(m match.Matcher, vp vulnerability.Provider, p pkg.Package) (matches []match.Match, ignoredMatches []match.IgnoredMatch, err error) {
+	// handle individual matcher panics
+	defer func() {
+		if e := recover(); e != nil {
+			err = match.NewFatalError(m.Type(), fmt.Errorf("%v at:\n%s", e, string(debug.Stack())))
 		}
-		for l, vulnIDs := range falsePositivesByLocation {
-			distroFalsePositivesByLocationPath[l] = append(distroFalsePositivesByLocationPath[l], vulnIDs...)
-		}
-	}
-
-	return distroFalsePositivesByLocationPath, nil
-}
-
-func getDistroFalsePositivesByLocation(s v5.ProviderStore, d *distro.Distro, p pkg.Package) (map[string][]string, error) {
-	result := make(map[string][]string)
-
-	if data, ok := p.Metadata.(pkg.ApkMetadata); ok {
-		entries, err := s.GetByDistro(d, p)
-		if err != nil {
-			return nil, err
-		}
-		for _, entry := range entries {
-			if entry.Constraint.String() == "< 0 (apk)" {
-				for _, f := range data.Files {
-					result[f.Path] = append(result[f.Path], entry.ID)
-				}
-			}
-		}
-
-		for _, upstreamPkg := range pkg.UpstreamPackages(p) {
-			entriesForUpstream, err := s.GetByDistro(d, upstreamPkg)
-			if err != nil {
-				return nil, err
-			}
-			for _, entry := range entriesForUpstream {
-				if entry.Constraint.String() == "< 0 (apk)" {
-					for _, f := range data.Files {
-						result[f.Path] = append(result[f.Path], entry.ID)
-					}
-				}
-			}
-		}
-	}
-
-	return result, nil
-}
-
-func filterMatchesUsingDistroFalsePositives(ms []match.Match, falsePositivesByLocation map[string][]string) []match.Match {
-	var result []match.Match
-	for _, m := range ms {
-		isFalsePositive := false
-
-		for _, l := range m.Package.Locations.ToSlice() {
-			if fpVulnIDs, ok := falsePositivesByLocation[l.RealPath]; ok {
-				if slices.Contains(fpVulnIDs, m.Vulnerability.ID) {
-					isFalsePositive = true
-					break
-				}
-
-				for _, relatedVulnerability := range m.Vulnerability.RelatedVulnerabilities {
-					if slices.Contains(fpVulnIDs, relatedVulnerability.ID) {
-						isFalsePositive = true
-						break
-					}
-				}
-			}
-		}
-
-		if !isFalsePositive {
-			result = append(result, m)
-		} else {
-			log.WithFields("vuln", m.Vulnerability.ID, "package", displayPackage(m.Package)).Trace("dropping false positive using distro security data")
-		}
-	}
-
-	return result
+	}()
+	return m.Match(vp, p)
 }
 
 func (m *VulnerabilityMatcher) findVEXMatches(context pkg.Context, remainingMatches *match.Matches, ignoredMatches []match.IgnoredMatch, progressMonitor *monitorWriter) (*match.Matches, []match.IgnoredMatch, error) {
@@ -307,11 +250,12 @@ func (m *VulnerabilityMatcher) findVEXMatches(context pkg.Context, remainingMatc
 	// note: this assumes that the diff can only be additive
 	diffIgnoredMatches := ignoredMatchesDiff(ignoredMatchesAfterVex, ignoredMatches)
 
-	updateVulnerabilityList(progressMonitor, diffMatches.Sorted(), diffIgnoredMatches, nil, m.Store)
+	updateVulnerabilityList(progressMonitor, diffMatches.Sorted(), diffIgnoredMatches, nil, m.VulnerabilityProvider)
 
 	return matchesAfterVex, ignoredMatchesAfterVex, nil
 }
 
+// applyIgnoreRules applies the user-provided ignore rules, splitting ignored matches into a separate set
 func (m *VulnerabilityMatcher) applyIgnoreRules(matches match.Matches) (match.Matches, []match.IgnoredMatch) {
 	var ignoredMatches []match.IgnoredMatch
 	if len(m.IgnoreRules) == 0 {
@@ -359,7 +303,7 @@ func (m *VulnerabilityMatcher) normalizeByCVE(match match.Match) match.Match {
 
 	ref := effectiveCVERecordRefs[0]
 
-	upstreamMetadata, err := m.Store.GetMetadata(ref.ID, ref.Namespace)
+	upstreamMetadata, err := m.VulnerabilityProvider.VulnerabilityMetadata(ref)
 	if err != nil {
 		log.WithFields("id", ref.ID, "namespace", ref.Namespace, "error", err).Warn("unable to fetch effective CVE metadata")
 		return match
@@ -381,11 +325,53 @@ func (m *VulnerabilityMatcher) normalizeByCVE(match match.Match) match.Match {
 	return match
 }
 
+// ignoreRulesByLocation implements match.IgnoreFilter to filter each matching
+// package that overlaps by location and have the same vulnerability ID (CVE)
+type ignoreRulesByLocation struct {
+	locationToIgnoreRules map[string][]match.IgnoreRule
+}
+
+func (i ignoreRulesByLocation) IgnoreMatch(m match.Match) []match.IgnoreRule {
+	for _, l := range m.Package.Locations.ToSlice() {
+		for _, rule := range i.locationToIgnoreRules[l.RealPath] {
+			if rule.Vulnerability == m.Vulnerability.ID {
+				return []match.IgnoreRule{rule}
+			}
+			for _, relatedVulnerability := range m.Vulnerability.RelatedVulnerabilities {
+				if rule.Vulnerability == relatedVulnerability.ID {
+					return []match.IgnoreRule{rule}
+				}
+			}
+		}
+	}
+	return nil
+}
+
+// ignoreMatchFilter creates an ignore filter based on the provided IgnoredMatches to filter out "the same"
+// vulnerabilities reported by other matchers based on overlapping file locations
+func ignoredMatchFilter(ignores []match.IgnoredMatch) match.IgnoreFilter {
+	out := ignoreRulesByLocation{locationToIgnoreRules: map[string][]match.IgnoreRule{}}
+	for _, ignore := range ignores {
+		// TODO should this be syftPkg.FileOwner interface or similar?
+		if m, ok := ignore.Package.Metadata.(pkg.ApkMetadata); ok {
+			for _, f := range m.Files {
+				out.locationToIgnoreRules[f.Path] = append(out.locationToIgnoreRules[f.Path], ignore.AppliedIgnoreRules...)
+			}
+		}
+	}
+	return out
+}
+
 func displayPackage(p pkg.Package) string {
 	if p.PURL != "" {
 		return p.PURL
 	}
-	return fmt.Sprintf("%s@%s (%s)", p.Name, p.Version, p.Type)
+	ty := p.Type
+	if p.Type == "" {
+		ty = "unknown"
+	}
+
+	return fmt.Sprintf("%s@%s (type=%s)", p.Name, p.Version, ty)
 }
 
 func ignoredMatchesDiff(subject []match.IgnoredMatch, other []match.IgnoredMatch) []match.IgnoredMatch {
@@ -394,12 +380,12 @@ func ignoredMatchesDiff(subject []match.IgnoredMatch, other []match.IgnoredMatch
 
 	otherMap := make(map[match.Fingerprint]struct{})
 	for _, a := range other {
-		otherMap[a.Match.Fingerprint()] = struct{}{}
+		otherMap[a.Fingerprint()] = struct{}{}
 	}
 
 	var diff []match.IgnoredMatch
 	for _, b := range subject {
-		if _, ok := otherMap[b.Match.Fingerprint()]; !ok {
+		if _, ok := otherMap[b.Fingerprint()]; !ok {
 			diff = append(diff, b)
 		}
 	}
@@ -407,9 +393,9 @@ func ignoredMatchesDiff(subject []match.IgnoredMatch, other []match.IgnoredMatch
 	return diff
 }
 
-func newMatcherIndex(matchers []matcher.Matcher) (map[syftPkg.Type][]matcher.Matcher, matcher.Matcher) {
-	matcherIndex := make(map[syftPkg.Type][]matcher.Matcher)
-	var defaultMatcher matcher.Matcher
+func newMatcherIndex(matchers []match.Matcher) (map[syftPkg.Type][]match.Matcher, match.Matcher) {
+	matcherIndex := make(map[syftPkg.Type][]match.Matcher)
+	var defaultMatcher match.Matcher
 	for _, m := range matchers {
 		if m.Type() == match.StockMatcher {
 			defaultMatcher = m
@@ -417,11 +403,11 @@ func newMatcherIndex(matchers []matcher.Matcher) (map[syftPkg.Type][]matcher.Mat
 		}
 		for _, t := range m.PackageTypes() {
 			if _, ok := matcherIndex[t]; !ok {
-				matcherIndex[t] = make([]matcher.Matcher, 0)
+				matcherIndex[t] = make([]match.Matcher, 0)
 			}
 
 			matcherIndex[t] = append(matcherIndex[t], m)
-			log.Debugf("adding matcher: %+v", t)
+			log.Tracef("adding matcher: %+v", t)
 		}
 	}
 
@@ -432,12 +418,12 @@ func isCVE(id string) bool {
 	return strings.HasPrefix(strings.ToLower(id), "cve-")
 }
 
-func HasSeverityAtOrAbove(store v5.VulnerabilityMetadataProvider, severity vulnerability.Severity, matches match.Matches) bool {
+func hasSeverityAtOrAbove(store vulnerability.MetadataProvider, severity vulnerability.Severity, matches match.Matches) bool {
 	if severity == vulnerability.UnknownSeverity {
 		return false
 	}
 	for m := range matches.Enumerate() {
-		metadata, err := store.GetMetadata(m.Vulnerability.ID, m.Vulnerability.Namespace)
+		metadata, err := store.VulnerabilityMetadata(m.Vulnerability.Reference)
 		if err != nil {
 			continue
 		}
@@ -469,9 +455,9 @@ func logListSummary(vl *monitorWriter) {
 	}
 }
 
-func updateVulnerabilityList(mon *monitorWriter, matches []match.Match, ignores []match.IgnoredMatch, dropped []match.IgnoredMatch, metadataProvider v5.VulnerabilityMetadataProvider) {
+func updateVulnerabilityList(mon *monitorWriter, matches []match.Match, ignores []match.IgnoredMatch, dropped []match.IgnoredMatch, metadataProvider vulnerability.MetadataProvider) {
 	for _, m := range matches {
-		metadata, err := metadataProvider.GetMetadata(m.Vulnerability.ID, m.Vulnerability.Namespace)
+		metadata, err := metadataProvider.VulnerabilityMetadata(m.Vulnerability.Reference)
 		if err != nil || metadata == nil {
 			mon.BySeverity[vulnerability.UnknownSeverity].Increment()
 			continue
