@@ -1,6 +1,7 @@
 package grype
 
 import (
+	"errors"
 	"testing"
 
 	"github.com/google/go-cmp/cmp"
@@ -10,20 +11,21 @@ import (
 	"github.com/stretchr/testify/require"
 	"github.com/wagoodman/go-partybus"
 
-	v5 "github.com/anchore/grype/grype/db/v5"
-	"github.com/anchore/grype/grype/db/v5/matcher"
-	"github.com/anchore/grype/grype/db/v5/matcher/ruby"
-	"github.com/anchore/grype/grype/db/v5/search"
 	"github.com/anchore/grype/grype/distro"
 	"github.com/anchore/grype/grype/event"
 	"github.com/anchore/grype/grype/event/monitor"
 	"github.com/anchore/grype/grype/grypeerr"
 	"github.com/anchore/grype/grype/match"
+	"github.com/anchore/grype/grype/matcher"
+	"github.com/anchore/grype/grype/matcher/apk"
+	matcherMock "github.com/anchore/grype/grype/matcher/mock"
+	"github.com/anchore/grype/grype/matcher/ruby"
 	"github.com/anchore/grype/grype/pkg"
 	"github.com/anchore/grype/grype/pkg/qualifier"
 	"github.com/anchore/grype/grype/version"
 	"github.com/anchore/grype/grype/vex"
 	"github.com/anchore/grype/grype/vulnerability"
+	"github.com/anchore/grype/grype/vulnerability/mock"
 	"github.com/anchore/grype/internal/bus"
 	"github.com/anchore/syft/syft/cpe"
 	"github.com/anchore/syft/syft/file"
@@ -32,182 +34,89 @@ import (
 	"github.com/anchore/syft/syft/source"
 )
 
-type ack interface {
-	v5.VulnerabilityStoreReader
-	v5.VulnerabilityMetadataStoreReader
-	v5.VulnerabilityMatchExclusionStoreReader
-}
-
-var _ ack = (*mockStore)(nil)
-
-type mockStore struct {
-	vulnerabilities map[string]map[string][]v5.Vulnerability
-	metadata        map[string]map[string]*v5.VulnerabilityMetadata
-}
-
-func (d *mockStore) GetVulnerabilityMatchExclusion(id string) ([]v5.VulnerabilityMatchExclusion, error) {
-	// panic("implement me")
-	return nil, nil
-}
-
-// A mockStoreStubFn takes a reference to a mockStore and mutates it to contain
-// a set of prescribed test data.
-type mockStoreStubFn func(*mockStore)
-
-// newMockStore returns a new mock implementation of a Grype database store. If
-// the stubFn parameter is not set to nil, the given stub function will be used
-// to modify the data in the mock store, such as in preparation for tests.
-func newMockStore(stubFn mockStoreStubFn) *mockStore {
-	d := &mockStore{
-		vulnerabilities: make(map[string]map[string][]v5.Vulnerability),
-		metadata:        make(map[string]map[string]*v5.VulnerabilityMetadata),
-	}
-	if stubFn != nil {
-		stubFn(d)
-	}
-	return d
-}
-
-func (d *mockStore) GetVulnerabilityMetadata(id, namespace string) (*v5.VulnerabilityMetadata, error) {
-	return d.metadata[id][namespace], nil
-}
-
-func (d *mockStore) GetAllVulnerabilityMetadata() (*[]v5.VulnerabilityMetadata, error) {
-	panic("implement me")
-}
-
-func (d *mockStore) GetVulnerability(namespace, id string) ([]v5.Vulnerability, error) {
-	var results []v5.Vulnerability
-	for _, vulns := range d.vulnerabilities[namespace] {
-		for _, vuln := range vulns {
-			if vuln.ID == id {
-				results = append(results, vuln)
-			}
-		}
-	}
-	return results, nil
-}
-
-func (d *mockStore) SearchForVulnerabilities(namespace, name string) ([]v5.Vulnerability, error) {
-	return d.vulnerabilities[namespace][name], nil
-}
-
-func (d *mockStore) GetAllVulnerabilities() (*[]v5.Vulnerability, error) {
-	panic("implement me")
-}
-
-func (d *mockStore) GetVulnerabilityNamespaces() ([]string, error) {
-	keys := make([]string, 0, len(d.vulnerabilities))
-	for k := range d.vulnerabilities {
-		keys = append(keys, k)
-	}
-
-	return keys, nil
-}
-
-func defaultStubFn(d *mockStore) {
-	// METADATA /////////////////////////////////////////////////////////////////////////////////
-	d.metadata["CVE-2014-fake-1"] = map[string]*v5.VulnerabilityMetadata{
-		"debian:distro:debian:8": {
-			ID:        "CVE-2014-fake-1",
-			Namespace: "debian:distro:debian:8",
-			Severity:  "medium",
-		},
-	}
-
-	d.metadata["GHSA-2014-fake-3"] = map[string]*v5.VulnerabilityMetadata{
-		"github:language:ruby": {
-			ID:        "GHSA-2014-fake-3",
-			Namespace: "github:language:ruby",
-			Severity:  "medium",
-		},
-	}
-
-	d.metadata["CVE-2014-fake-3"] = map[string]*v5.VulnerabilityMetadata{
-		"nvd:cpe": {
-			ID:        "CVE-2014-fake-3",
-			Namespace: "nvd:cpe",
-			Severity:  "critical",
-		},
-	}
-
-	// VULNERABILITIES ///////////////////////////////////////////////////////////////////////////
-	d.vulnerabilities["debian:distro:debian:8"] = map[string][]v5.Vulnerability{
-		"neutron": {
-			{
-				PackageName:       "neutron",
-				Namespace:         "debian:distro:debian:8",
-				VersionConstraint: "< 2014.1.3-6",
-				ID:                "CVE-2014-fake-1",
-				VersionFormat:     "deb",
-			},
-			{
-				PackageName:       "neutron",
-				Namespace:         "debian:distro:debian:8",
-				VersionConstraint: "< 2013.0.2-1",
-				ID:                "CVE-2013-fake-2",
-				VersionFormat:     "deb",
+func testVulnerabilities() []vulnerability.Vulnerability {
+	return []vulnerability.Vulnerability{
+		{
+			Reference: vulnerability.Reference{
+				ID:        "CVE-2014-fake-1",
+				Namespace: "debian:distro:debian:8",
+				Internal: vulnerability.Metadata{
+					Severity: "medium",
+				},
 			},
+			PackageName: "neutron",
+			Constraint:  version.MustGetConstraint("< 2014.1.3-6", version.DebFormat),
 		},
-	}
-	d.vulnerabilities["github:language:ruby"] = map[string][]v5.Vulnerability{
-		"activerecord": {
-			{
-				PackageName:       "activerecord",
-				Namespace:         "github:language:ruby",
-				VersionConstraint: "< 3.7.6",
-				ID:                "GHSA-2014-fake-3",
-				VersionFormat:     "unknown",
-				RelatedVulnerabilities: []v5.VulnerabilityReference{
-					{
-						ID:        "CVE-2014-fake-3",
-						Namespace: "nvd:cpe",
-					},
+		{
+			Reference: vulnerability.Reference{
+				ID:        "CVE-2013-fake-2",
+				Namespace: "debian:distro:debian:8",
+			},
+			PackageName: "neutron",
+			Constraint:  version.MustGetConstraint("< 2013.0.2-1", version.DebFormat),
+		},
+		{
+			Reference: vulnerability.Reference{
+				ID:        "GHSA-2014-fake-3",
+				Namespace: "github:language:ruby",
+				Internal: vulnerability.Metadata{
+					Severity: "medium",
+				},
+			},
+			PackageName: "activerecord",
+			Constraint:  version.MustGetConstraint("< 3.7.6", version.UnknownFormat),
+			RelatedVulnerabilities: []vulnerability.Reference{
+				{
+					ID:        "CVE-2014-fake-3",
+					Namespace: "nvd:cpe",
 				},
 			},
 		},
-	}
-	d.vulnerabilities["nvd:cpe"] = map[string][]v5.Vulnerability{
-		"activerecord": {
-			{
-				PackageName:       "activerecord",
-				Namespace:         "nvd:cpe",
-				VersionConstraint: "< 3.7.6",
-				ID:                "CVE-2014-fake-3",
-				VersionFormat:     "unknown",
-				CPEs: []string{
-					"cpe:2.3:*:activerecord:activerecord:*:*:*:*:*:rails:*:*",
+		{
+			Reference: vulnerability.Reference{
+				ID:        "CVE-2014-fake-3",
+				Namespace: "nvd:cpe",
+				Internal: vulnerability.Metadata{
+					Severity: "critical",
 				},
 			},
-			{
-				PackageName:       "activerecord",
-				Namespace:         "nvd:cpe",
-				VersionConstraint: "< 3.7.4",
-				ID:                "CVE-2014-fake-4",
-				VersionFormat:     "unknown",
-				CPEs: []string{
-					"cpe:2.3:*:activerecord:activerecord:*:*:something:*:*:ruby:*:*",
-				},
+			PackageName: "activerecord",
+			Constraint:  version.MustGetConstraint("< 3.7.6", version.UnknownFormat),
+			CPEs: []cpe.CPE{
+				cpe.Must("cpe:2.3:*:activerecord:activerecord:*:*:*:*:*:rails:*:*", ""),
 			},
-			{
-				PackageName:       "activerecord",
-				Namespace:         "nvd:cpe",
-				VersionConstraint: "= 4.0.1",
-				ID:                "CVE-2014-fake-5",
-				VersionFormat:     "unknown",
-				CPEs: []string{
-					"cpe:2.3:*:couldntgetthisrightcouldyou:activerecord:4.0.1:*:*:*:*:*:*:*",
-				},
+		},
+		{
+			Reference: vulnerability.Reference{
+				ID:        "CVE-2014-fake-4",
+				Namespace: "nvd:cpe",
 			},
-			{
-				PackageName:       "activerecord",
-				Namespace:         "nvd:cpe",
-				VersionConstraint: "< 98SP3",
-				ID:                "CVE-2014-fake-6",
-				VersionFormat:     "unknown",
-				CPEs: []string{
-					"cpe:2.3:*:awesome:awesome:*:*:*:*:*:*:*:*",
-				},
+			PackageName: "activerecord",
+			Constraint:  version.MustGetConstraint("< 3.7.4", version.UnknownFormat),
+			CPEs: []cpe.CPE{
+				cpe.Must("cpe:2.3:*:activerecord:activerecord:*:*:something:*:*:ruby:*:*", ""),
+			},
+		},
+		{
+			Reference: vulnerability.Reference{
+				ID:        "CVE-2014-fake-5",
+				Namespace: "nvd:cpe",
+			},
+			PackageName: "activerecord",
+			Constraint:  version.MustGetConstraint("= 4.0.1", version.UnknownFormat),
+			CPEs: []cpe.CPE{
+				cpe.Must("cpe:2.3:*:couldntgetthisrightcouldyou:activerecord:4.0.1:*:*:*:*:*:*:*", ""),
+			},
+		},
+		{
+			Reference: vulnerability.Reference{
+				ID:        "CVE-2014-fake-6",
+				Namespace: "nvd:cpe",
+			},
+			PackageName: "activerecord",
+			Constraint:  version.MustGetConstraint("< 98SP3", version.UnknownFormat),
+			CPEs: []cpe.CPE{
+				cpe.Must("cpe:2.3:*:awesome:awesome:*:*:*:*:*:*:*:*", ""),
 			},
 		},
 	}
@@ -269,7 +178,7 @@ func Test_HasSeverityAtOrAbove(t *testing.T) {
 		},
 	}
 
-	metadataProvider := v5.NewVulnerabilityMetadataProvider(newMockStore(defaultStubFn))
+	metadataProvider := mock.VulnerabilityProvider(testVulnerabilities()...)
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
@@ -282,7 +191,7 @@ func Test_HasSeverityAtOrAbove(t *testing.T) {
 				failOnSeverity = sev
 			}
 
-			actual := HasSeverityAtOrAbove(metadataProvider, failOnSeverity, test.matches)
+			actual := hasSeverityAtOrAbove(metadataProvider, failOnSeverity, test.matches)
 
 			if test.expectedResult != actual {
 				t.Errorf("expected: %v got : %v", test.expectedResult, actual)
@@ -292,14 +201,7 @@ func Test_HasSeverityAtOrAbove(t *testing.T) {
 }
 
 func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
-	mkStr := newMockStore(defaultStubFn)
-	vp, err := v5.NewVulnerabilityProvider(mkStr)
-	require.NoError(t, err)
-	str := v5.ProviderStore{
-		VulnerabilityProvider:         vp,
-		VulnerabilityMetadataProvider: v5.NewVulnerabilityMetadataProvider(mkStr),
-		ExclusionProvider:             v5.NewMatchExclusionProvider(mkStr),
-	}
+	vp := mock.VulnerabilityProvider(testVulnerabilities()...)
 
 	neutron2013Pkg := pkg.Package{
 		ID:      pkg.ID(uuid.NewString()),
@@ -328,8 +230,7 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 	}
 
 	type fields struct {
-		Store          v5.ProviderStore
-		Matchers       []matcher.Matcher
+		Matchers       []match.Matcher
 		IgnoreRules    []match.IgnoreRule
 		FailSeverity   *vulnerability.Severity
 		NormalizeByCVE bool
@@ -351,7 +252,6 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 		{
 			name: "no matches",
 			fields: fields{
-				Store:    str,
 				Matchers: matcher.NewDefaultMatchers(matcher.Config{}),
 			},
 			args: args{
@@ -374,7 +274,6 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 		{
 			name: "matches by exact-direct match (OS)",
 			fields: fields{
-				Store:    str,
 				Matchers: matcher.NewDefaultMatchers(matcher.Config{}),
 			},
 			args: args{
@@ -426,7 +325,6 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 		{
 			name: "fail on severity threshold",
 			fields: fields{
-				Store:    str,
 				Matchers: matcher.NewDefaultMatchers(matcher.Config{}),
 				FailSeverity: func() *vulnerability.Severity {
 					x := vulnerability.LowSeverity
@@ -482,7 +380,6 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 		{
 			name: "pass on severity threshold with VEX",
 			fields: fields{
-				Store:    str,
 				Matchers: matcher.NewDefaultMatchers(matcher.Config{}),
 				FailSeverity: func() *vulnerability.Severity {
 					x := vulnerability.LowSeverity
@@ -565,7 +462,6 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 		{
 			name: "matches by exact-direct match (language)",
 			fields: fields{
-				Store: str,
 				Matchers: matcher.NewDefaultMatchers(matcher.Config{
 					Ruby: ruby.MatcherConfig{
 						UseCPEs: true,
@@ -597,17 +493,17 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 					Details: match.Details{
 						{
 							Type: match.CPEMatch,
-							SearchedBy: search.CPEParameters{
+							SearchedBy: match.CPEParameters{
 								Namespace: "nvd:cpe",
 								CPEs: []string{
 									"cpe:2.3:*:activerecord:activerecord:3.7.5:*:*:*:*:rails:*:*",
 								},
-								Package: search.CPEPackageParameter{
+								Package: match.CPEPackageParameter{
 									Name:    "activerecord",
 									Version: "3.7.5",
 								},
 							},
-							Found: search.CPEResult{
+							Found: match.CPEResult{
 								VulnerabilityID:   "CVE-2014-fake-3",
 								VersionConstraint: "< 3.7.6 (unknown)",
 								CPEs: []string{
@@ -662,7 +558,6 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 		{
 			name: "normalize by cve",
 			fields: fields{
-				Store: str,
 				Matchers: matcher.NewDefaultMatchers(
 					matcher.Config{
 						Ruby: ruby.MatcherConfig{
@@ -717,17 +612,17 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 						},
 						{
 							Type: match.CPEMatch,
-							SearchedBy: search.CPEParameters{
+							SearchedBy: match.CPEParameters{
 								Namespace: "nvd:cpe",
 								CPEs: []string{
 									"cpe:2.3:*:activerecord:activerecord:3.7.5:*:*:*:*:rails:*:*",
 								},
-								Package: search.CPEPackageParameter{
+								Package: match.CPEPackageParameter{
 									Name:    "activerecord",
 									Version: "3.7.5",
 								},
 							},
-							Found: search.CPEResult{
+							Found: match.CPEResult{
 								VulnerabilityID:   "CVE-2014-fake-3",
 								VersionConstraint: "< 3.7.6 (unknown)",
 								CPEs: []string{
@@ -746,7 +641,6 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 		{
 			name: "normalize by cve -- ignore GHSA",
 			fields: fields{
-				Store: str,
 				Matchers: matcher.NewDefaultMatchers(
 					matcher.Config{
 						Ruby: ruby.MatcherConfig{
@@ -786,17 +680,17 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 					Details: match.Details{
 						{
 							Type: match.CPEMatch,
-							SearchedBy: search.CPEParameters{
+							SearchedBy: match.CPEParameters{
 								Namespace: "nvd:cpe",
 								CPEs: []string{
 									"cpe:2.3:*:activerecord:activerecord:3.7.5:*:*:*:*:rails:*:*",
 								},
-								Package: search.CPEPackageParameter{
+								Package: match.CPEPackageParameter{
 									Name:    "activerecord",
 									Version: "3.7.5",
 								},
 							},
-							Found: search.CPEResult{
+							Found: match.CPEResult{
 								VulnerabilityID:   "CVE-2014-fake-3",
 								VersionConstraint: "< 3.7.6 (unknown)",
 								CPEs: []string{
@@ -859,7 +753,6 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 		{
 			name: "normalize by cve -- ignore CVE",
 			fields: fields{
-				Store: str,
 				Matchers: matcher.NewDefaultMatchers(
 					matcher.Config{
 						Ruby: ruby.MatcherConfig{
@@ -902,17 +795,17 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 						Details: match.Details{
 							{
 								Type: match.CPEMatch,
-								SearchedBy: search.CPEParameters{
+								SearchedBy: match.CPEParameters{
 									Namespace: "nvd:cpe",
 									CPEs: []string{
 										"cpe:2.3:*:activerecord:activerecord:3.7.5:*:*:*:*:rails:*:*",
 									},
-									Package: search.CPEPackageParameter{
+									Package: match.CPEPackageParameter{
 										Name:    "activerecord",
 										Version: "3.7.5",
 									},
 								},
-								Found: search.CPEResult{
+								Found: match.CPEResult{
 									VulnerabilityID:   "CVE-2014-fake-3",
 									VersionConstraint: "< 3.7.6 (unknown)",
 									CPEs: []string{
@@ -979,7 +872,6 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 		{
 			name: "ignore CVE (not normalized by CVE)",
 			fields: fields{
-				Store: str,
 				Matchers: matcher.NewDefaultMatchers(matcher.Config{
 					Ruby: ruby.MatcherConfig{
 						UseCPEs: true,
@@ -1059,17 +951,17 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 						Details: match.Details{
 							{
 								Type: match.CPEMatch,
-								SearchedBy: search.CPEParameters{
+								SearchedBy: match.CPEParameters{
 									Namespace: "nvd:cpe",
 									CPEs: []string{
 										"cpe:2.3:*:activerecord:activerecord:3.7.5:*:*:*:*:rails:*:*",
 									},
-									Package: search.CPEPackageParameter{
+									Package: match.CPEPackageParameter{
 										Name:    "activerecord",
 										Version: "3.7.5",
 									},
 								},
-								Found: search.CPEResult{
+								Found: match.CPEResult{
 									VulnerabilityID:   "CVE-2014-fake-3",
 									VersionConstraint: "< 3.7.6 (unknown)",
 									CPEs: []string{
@@ -1086,15 +978,16 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 			wantErr: nil,
 		},
 	}
+
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
 			m := &VulnerabilityMatcher{
-				Store:          tt.fields.Store,
-				Matchers:       tt.fields.Matchers,
-				IgnoreRules:    tt.fields.IgnoreRules,
-				FailSeverity:   tt.fields.FailSeverity,
-				NormalizeByCVE: tt.fields.NormalizeByCVE,
-				VexProcessor:   tt.fields.VexProcessor,
+				VulnerabilityProvider: vp,
+				Matchers:              tt.fields.Matchers,
+				IgnoreRules:           tt.fields.IgnoreRules,
+				FailSeverity:          tt.fields.FailSeverity,
+				NormalizeByCVE:        tt.fields.NormalizeByCVE,
+				VexProcessor:          tt.fields.VexProcessor,
 			}
 
 			listener := &busListener{}
@@ -1111,9 +1004,11 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 			}
 
 			var opts = []cmp.Option{
+				cmpopts.EquateEmpty(),
 				cmpopts.IgnoreUnexported(match.Match{}),
 				cmpopts.IgnoreFields(vulnerability.Vulnerability{}, "Constraint"),
-				cmpopts.IgnoreFields(pkg.Package{}, "Locations"),
+				cmpopts.IgnoreFields(vulnerability.Reference{}, "Internal"),
+				cmpopts.IgnoreFields(pkg.Package{}, "Locations", "Distro"),
 				cmpopts.IgnoreUnexported(match.IgnoredMatch{}),
 			}
 
@@ -1131,21 +1026,75 @@ func TestVulnerabilityMatcher_FindMatches(t *testing.T) {
 	}
 }
 
+func Test_fatalErrors(t *testing.T) {
+	tests := []struct {
+		name        string
+		matcherFunc matcherMock.MatchFunc
+		assertErr   assert.ErrorAssertionFunc
+	}{
+		{
+			name: "no error",
+			matcherFunc: func(_ vulnerability.Provider, _ pkg.Package) ([]match.Match, []match.IgnoredMatch, error) {
+				return nil, nil, nil
+			},
+			assertErr: assert.NoError,
+		},
+		{
+			name: "non-fatal error",
+			matcherFunc: func(_ vulnerability.Provider, _ pkg.Package) ([]match.Match, []match.IgnoredMatch, error) {
+				return nil, nil, errors.New("some error")
+			},
+			assertErr: assert.NoError,
+		},
+		{
+			name: "fatal error",
+			matcherFunc: func(_ vulnerability.Provider, _ pkg.Package) ([]match.Match, []match.IgnoredMatch, error) {
+				return nil, nil, match.NewFatalError(match.UnknownMatcherType, errors.New("some error"))
+			},
+			assertErr: assert.Error,
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			m := &VulnerabilityMatcher{
+				Matchers: []match.Matcher{matcherMock.New(syftPkg.JavaPkg, tt.matcherFunc)},
+			}
+
+			_, _, err := m.FindMatches([]pkg.Package{
+				{
+					Name:    "foo",
+					Version: "1.2.3",
+					Type:    syftPkg.JavaPkg,
+				},
+			},
+				pkg.Context{
+					Distro: &linux.Release{
+						ID:        "debian",
+						VersionID: "8",
+					},
+				},
+			)
+
+			tt.assertErr(t, err)
+		})
+	}
+}
+
 func Test_indexFalsePositivesByLocation(t *testing.T) {
 	cases := []struct {
 		name           string
-		d              distro.Distro
 		pkgs           []pkg.Package
-		stubFunc       mockStoreStubFn
+		vulns          []vulnerability.Vulnerability
 		expectedResult map[string][]string
 		errAssertion   assert.ErrorAssertionFunc
 	}{
 		{
 			name: "false positive in wolfi package adds index entry",
-			d:    distro.Distro{Type: distro.Wolfi},
 			pkgs: []pkg.Package{
 				{
-					Name: "foo",
+					Name:   "foo",
+					Distro: &distro.Distro{Type: distro.Wolfi},
 					Metadata: pkg.ApkMetadata{Files: []pkg.ApkFileRecord{
 						{
 							Path: "/bin/foo-binary",
@@ -1153,18 +1102,15 @@ func Test_indexFalsePositivesByLocation(t *testing.T) {
 					}},
 				},
 			},
-			stubFunc: func(d *mockStore) {
-				d.vulnerabilities["wolfi:distro:wolfi:rolling"] = map[string][]v5.Vulnerability{
-					"foo": {
-						{
-							ID:                "GHSA-2014-fake-3",
-							PackageName:       "foo",
-							Namespace:         "wolfi:distro:wolfi:rolling",
-							VersionConstraint: "< 0",
-							VersionFormat:     "apk",
-						},
+			vulns: []vulnerability.Vulnerability{
+				{
+					Reference: vulnerability.Reference{
+						ID:        "GHSA-2014-fake-3",
+						Namespace: "wolfi:distro:wolfi:rolling",
 					},
-				}
+					PackageName: "foo",
+					Constraint:  version.MustGetConstraint("< 0", version.ApkFormat),
+				},
 			},
 			expectedResult: map[string][]string{
 				"/bin/foo-binary": {"GHSA-2014-fake-3"},
@@ -1173,10 +1119,10 @@ func Test_indexFalsePositivesByLocation(t *testing.T) {
 		},
 		{
 			name: "false positive in wolfi subpackage adds index entry",
-			d:    distro.Distro{Type: distro.Wolfi},
 			pkgs: []pkg.Package{
 				{
-					Name: "subpackage-foo",
+					Name:   "subpackage-foo",
+					Distro: &distro.Distro{Type: distro.Wolfi},
 					Metadata: pkg.ApkMetadata{Files: []pkg.ApkFileRecord{
 						{
 							Path: "/bin/foo-subpackage-binary",
@@ -1189,18 +1135,15 @@ func Test_indexFalsePositivesByLocation(t *testing.T) {
 					},
 				},
 			},
-			stubFunc: func(d *mockStore) {
-				d.vulnerabilities["wolfi:distro:wolfi:rolling"] = map[string][]v5.Vulnerability{
-					"origin-foo": {
-						{
-							ID:                "GHSA-2014-fake-3",
-							PackageName:       "foo",
-							Namespace:         "wolfi:distro:wolfi:rolling",
-							VersionConstraint: "< 0",
-							VersionFormat:     "apk",
-						},
+			vulns: []vulnerability.Vulnerability{
+				{
+					Reference: vulnerability.Reference{
+						ID:        "GHSA-2014-fake-3",
+						Namespace: "wolfi:distro:wolfi:rolling",
 					},
-				}
+					PackageName: "origin-foo",
+					Constraint:  version.MustGetConstraint("< 0", version.ApkFormat),
+				},
 			},
 			expectedResult: map[string][]string{
 				"/bin/foo-subpackage-binary": {"GHSA-2014-fake-3"},
@@ -1209,10 +1152,10 @@ func Test_indexFalsePositivesByLocation(t *testing.T) {
 		},
 		{
 			name: "fixed vuln (not a false positive) in wolfi package",
-			d:    distro.Distro{Type: distro.Wolfi},
 			pkgs: []pkg.Package{
 				{
-					Name: "foo",
+					Name:   "foo",
+					Distro: &distro.Distro{Type: distro.Wolfi},
 					Metadata: pkg.ApkMetadata{Files: []pkg.ApkFileRecord{
 						{
 							Path: "/bin/foo-binary",
@@ -1220,28 +1163,25 @@ func Test_indexFalsePositivesByLocation(t *testing.T) {
 					}},
 				},
 			},
-			stubFunc: func(d *mockStore) {
-				d.vulnerabilities["wolfi:distro:wolfi:rolling"] = map[string][]v5.Vulnerability{
-					"foo": {
-						{
-							ID:                "GHSA-2014-fake-3",
-							PackageName:       "foo",
-							Namespace:         "wolfi:distro:wolfi:rolling",
-							VersionConstraint: "< 1.2.3-r4",
-							VersionFormat:     "apk",
-						},
+			vulns: []vulnerability.Vulnerability{
+				{
+					Reference: vulnerability.Reference{
+						ID:        "GHSA-2014-fake-3",
+						Namespace: "wolfi:distro:wolfi:rolling",
 					},
-				}
+					PackageName: "foo",
+					Constraint:  version.MustGetConstraint("< 1.2.3-r4", version.ApkFormat),
+				},
 			},
 			expectedResult: map[string][]string{},
 			errAssertion:   assert.NoError,
 		},
 		{
 			name: "no vuln data for wolfi package",
-			d:    distro.Distro{Type: distro.Wolfi},
 			pkgs: []pkg.Package{
 				{
-					Name: "foo",
+					Name:   "foo",
+					Distro: &distro.Distro{Type: distro.Wolfi},
 					Metadata: pkg.ApkMetadata{Files: []pkg.ApkFileRecord{
 						{
 							Path: "/bin/foo-binary",
@@ -1249,33 +1189,28 @@ func Test_indexFalsePositivesByLocation(t *testing.T) {
 					}},
 				},
 			},
-			stubFunc: func(d *mockStore) {
-				d.vulnerabilities["wolfi:distro:wolfi:rolling"] = map[string][]v5.Vulnerability{}
-			},
+			vulns:          []vulnerability.Vulnerability{},
 			expectedResult: map[string][]string{},
 			errAssertion:   assert.NoError,
 		},
 		{
 			name: "no files listed for a wolfi package",
-			d:    distro.Distro{Type: distro.Wolfi},
 			pkgs: []pkg.Package{
 				{
 					Name:     "foo",
+					Distro:   &distro.Distro{Type: distro.Wolfi},
 					Metadata: pkg.ApkMetadata{Files: nil},
 				},
 			},
-			stubFunc: func(d *mockStore) {
-				d.vulnerabilities["wolfi:distro:wolfi:rolling"] = map[string][]v5.Vulnerability{
-					"foo": {
-						{
-							ID:                "GHSA-2014-fake-3",
-							PackageName:       "foo",
-							Namespace:         "wolfi:distro:wolfi:rolling",
-							VersionConstraint: "< 0",
-							VersionFormat:     "apk",
-						},
+			vulns: []vulnerability.Vulnerability{
+				{
+					Reference: vulnerability.Reference{
+						ID:        "GHSA-2014-fake-3",
+						Namespace: "wolfi:distro:wolfi:rolling",
 					},
-				}
+					PackageName: "foo",
+					Constraint:  version.MustGetConstraint("< 0", version.ApkFormat),
+				},
 			},
 			expectedResult: map[string][]string{},
 			errAssertion:   assert.NoError,
@@ -1284,28 +1219,34 @@ func Test_indexFalsePositivesByLocation(t *testing.T) {
 
 	for _, tt := range cases {
 		t.Run(tt.name, func(t *testing.T) {
-			s := createMockStore(t, tt.stubFunc)
-			actualResult, err := indexFalsePositivesByLocation(&tt.d, tt.pkgs, s)
-			tt.errAssertion(t, err)
+			// create mock vulnerability provider
+			vp := mock.VulnerabilityProvider(tt.vulns...)
+			apkMatcher := &apk.Matcher{}
+
+			var allMatches []match.Match
+			var allIgnores []match.IgnoredMatch
+			for _, p := range tt.pkgs {
+				matches, ignores, err := apkMatcher.Match(vp, p)
+				require.NoError(t, err)
+				allMatches = append(allMatches, matches...)
+				allIgnores = append(allIgnores, ignores...)
+			}
+
+			actualResult := map[string][]string{}
+			for _, ignore := range allIgnores {
+				apkMetadata, ok := ignore.Package.Metadata.(pkg.ApkMetadata)
+				require.True(t, ok)
+				for _, f := range apkMetadata.Files {
+					for _, r := range ignore.AppliedIgnoreRules {
+						actualResult[f.Path] = append(actualResult[f.Path], r.Vulnerability)
+					}
+				}
+			}
 			assert.Equal(t, tt.expectedResult, actualResult)
 		})
 	}
 }
 
-func createMockStore(t *testing.T, fn mockStoreStubFn) v5.ProviderStore {
-	t.Helper()
-
-	mkStr := newMockStore(fn)
-	vp, err := v5.NewVulnerabilityProvider(mkStr)
-	require.NoError(t, err)
-
-	return v5.ProviderStore{
-		VulnerabilityProvider:         vp,
-		VulnerabilityMetadataProvider: v5.NewVulnerabilityMetadataProvider(mkStr),
-		ExclusionProvider:             v5.NewMatchExclusionProvider(mkStr),
-	}
-}
-
 func Test_filterMatchesUsingDistroFalsePositives(t *testing.T) {
 	cases := []struct {
 		name         string
@@ -1411,12 +1352,67 @@ func Test_filterMatchesUsingDistroFalsePositives(t *testing.T) {
 
 	for _, tt := range cases {
 		t.Run(tt.name, func(t *testing.T) {
-			actual := filterMatchesUsingDistroFalsePositives(tt.inputMatches, tt.fpIndex)
+			var allIgnores []match.IgnoredMatch
+			for path, cves := range tt.fpIndex {
+				for _, cve := range cves {
+					allIgnores = append(allIgnores, match.IgnoredMatch{
+						Match: match.Match{
+							Package: pkg.Package{
+								Metadata: pkg.ApkMetadata{
+									Files: []pkg.ApkFileRecord{
+										{
+											Path: path,
+										},
+									},
+								},
+							},
+						},
+						AppliedIgnoreRules: []match.IgnoreRule{
+							{
+								Vulnerability: cve,
+							},
+						},
+					})
+				}
+			}
+
+			filter := ignoredMatchFilter(allIgnores)
+
+			actual, _ := match.ApplyIgnoreFilters(tt.inputMatches, filter)
+
 			assert.Equal(t, tt.expected, actual)
 		})
 	}
 }
 
+type panicyMatcher struct {
+	matcherType match.MatcherType
+}
+
+func (m *panicyMatcher) PackageTypes() []syftPkg.Type {
+	return nil
+}
+
+func (m *panicyMatcher) Type() match.MatcherType {
+	return m.matcherType
+}
+
+func (m *panicyMatcher) Match(_ vulnerability.Provider, _ pkg.Package) ([]match.Match, []match.IgnoredMatch, error) {
+	panic("test panic message")
+}
+
+func TestCallMatcherSafely_RecoverFromPanic(t *testing.T) {
+	matcher := &panicyMatcher{
+		matcherType: "test-matcher",
+	}
+	_, _, err := callMatcherSafely(matcher, nil, pkg.Package{})
+
+	require.Error(t, err)
+	assert.True(t, match.IsFatalError(err))
+	require.Contains(t, err.Error(), "test panic message", "missing message")
+	require.Contains(t, err.Error(), "test-matcher", "missing matcher name")
+}
+
 type busListener struct {
 	matching monitor.Matching
 }
