@@ -12,6 +12,7 @@ import (
 	"os/exec"
 	"path"
 	"path/filepath"
+	"slices"
 	"strings"
 	"testing"
 
@@ -144,8 +145,8 @@ func TestFileCatalog_Open(t *testing.T) {
 
 	require.Len(t, entries, 1)
 
-	opener := func() io.ReadCloser {
-		return io.NopCloser(entries[0].Reader)
+	opener := func() (io.ReadCloser, error) {
+		return io.NopCloser(entries[0].Reader), nil
 	}
 
 	catalog := NewFileCatalog()
@@ -584,6 +585,9 @@ func TestFileCatalog_GetByBasenameGlob(t *testing.T) {
 			if err != nil {
 				return
 			}
+			sortPath := func(a, b filetree.IndexEntry) int { return strings.Compare(a.Path, b.Path) }
+			slices.SortFunc(actual, sortPath)
+			slices.SortFunc(tt.want, sortPath)
 			if d := cmp.Diff(tt.want, actual,
 				cmpopts.EquateEmpty(),
 				cmpopts.IgnoreUnexported(file.Reference{}),
@@ -736,8 +740,11 @@ func TestFileCatalog_GetBasenames(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			actual := fileCatalog.Basenames()
-			assert.ElementsMatchf(t, tt.want, actual, "diff: %s", cmp.Diff(tt.want, actual))
+			for _, name := range tt.want {
+				e, err := fileCatalog.GetByBasenameGlob(name)
+				require.NoError(t, err)
+				require.NotEmpty(t, e)
+			}
 		})
 	}
 }
