@@ -6,12 +6,14 @@ import (
 )
 
 type File struct {
-	ID       string             `json:"id"`
-	Location file.Coordinates   `json:"location"`
-	Metadata *FileMetadataEntry `json:"metadata,omitempty"`
-	Contents string             `json:"contents,omitempty"`
-	Digests  []file.Digest      `json:"digests,omitempty"`
-	Licenses []FileLicense      `json:"licenses,omitempty"`
+	ID         string             `json:"id"`
+	Location   file.Coordinates   `json:"location"`
+	Metadata   *FileMetadataEntry `json:"metadata,omitempty"`
+	Contents   string             `json:"contents,omitempty"`
+	Digests    []file.Digest      `json:"digests,omitempty"`
+	Licenses   []FileLicense      `json:"licenses,omitempty"`
+	Executable *file.Executable   `json:"executable,omitempty"`
+	Unknowns   []string           `json:"unknowns,omitempty"`
 }
 
 type FileMetadataEntry struct {
