@@ -5,8 +5,7 @@ import (
 	"path/filepath"
 	"strings"
 
-	"github.com/mitchellh/go-homedir"
-
+	"github.com/anchore/go-homedir"
 	"github.com/anchore/syft/internal/log"
 )
 
